// RW Bumpmaster
// ==============
// Imageboard Retrieval & Archiving Library
// -Part of the RW Foundations suite of libraries

// Filename: ./include/curator/bumpmaster/bumpmaster.hpp
// -Declarations of the baseline RW Bumpmaster sub-library

#pragma once

#include "board.hpp"
#include "catalog.hpp"
#include "page.hpp"
#include "post.hpp"
#include "site.hpp"
#include "thread.hpp"

#include "incls_all.hpp"

namespace rw::curator::bumpmaster {

//------------------------------------------------------------------------------
/**
 */
class Bumpmaster_b : public RW {
 public:
  Bumpmaster_b();                                    // default ctor
  explicit Bumpmaster_b(std::string const& handle);  // tag-parm'd ctor
};

//------------------------------------------------------------------------------
/**
 */
class Bumpmaster : public Bumpmaster_b {
 public:
  Bumpmaster();                                    // default ctor
  explicit Bumpmaster(std::string const& handle);  // tag-parm'd ctor

  inline auto board() const -> Board& { return board_; }
  inline auto catalog() const -> Catalog& { return catalog_; }
  inline auto page() const -> Page& { return page_; }
  inline auto post() const -> Post& { return post_; }
  inline auto site() const -> Site& { return site_; }
  inline auto thread() const -> Thread& { return thread_; }

  inline auto waifusearch() const -> Waifusearch& { return waifusearch_; }

 private:
  /**
   */
  void preflight();

  unique_ptr<Board_b>   _board_{make_unique<Board_b>(t(), "bumpmaster_board")};
  unique_ptr<Catalog_b> _catalog_{
      make_unique<Catalog_b>(t(), "bumpmaster_catalog")};
  unique_ptr<Page_b>   _page_{make_unique<Page_b>(t(), "bumpmaster_page")};
  unique_ptr<Post_b>   _post_{make_unique<Post_b>(t(), "bumpmaster_post")};
  unique_ptr<Site_b>   _site_{make_unique<Site_b>(t(), "bumpmaster_site")};
  unique_ptr<Thread_b> _thread_{
      make_unique<Thread_b>(t(), "bumpmaster_thread")};

  unique_ptr<Waifusearch_b> _waifusearch_{
      make_unique<Waifusearch_b>("bumpmaster_waifusearch")};

  Board&   board_{reference_to<Board>(_board_.get())};
  Catalog& catalog_{reference_to<Catalog>(_catalog_.get())};
  Page&    page_{reference_to<Page>(_page_.get())};
  Post&    post_{reference_to<Post>(_post_.get())};
  Site&    site_{reference_to<Site>(_site_.get())};
  Thread&  thread_{reference_to<Thread>(_thread_.get())};

  Waifusearch& waifusearch_{reference_to<Waifusearch>(_waifusearch_.get())};

///////
// Dolldrop/Dollnet is meant for network-enabled RW projects only
#if __has_include(<curl_easy.h>)

 public:
  inline auto dolldrop() const -> Dolldrop& { return dolldrop_; }

 private:
  unique_ptr<Dolldrop_b> _dolldrop_{
      make_unique<Dolldrop_b>(t(), "bumpmaster_dolldrop")};

  Dolldrop& dolldrop_{reference_to<Dolldrop>(_dolldrop_.get())};

#endif  // __has_include(<curl_easy.h>)
};

}  // namespace rw::curator::bumpmaster

//    --=^=--

// "______"

// rw_curator_lib.v220108
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
